/* ============================================================================
 * AMES Wholesale Power Market Test Bed (Java): A Free Open-Source Test-Bed
 *         for the Agent-based Modeling of Electricity Systems
 * ============================================================================
 *
 * (C) Copyright 2013, by Hongyan Li, Junjie Sun, Sean L. Mooney and Leigh Tesfatsion
 *
 *    Homepage: http://www.econ.iastate.edu/tesfatsi/testMarketHome.htm
 *
 * LICENSING TERMS
 * The AMES Market Package is licensed by the copyright holders (Junjie Sun,
 * Hongyan Li, Sean L. Mooney, and Leigh Tesfatsion) as free open-source software under the
 * terms of the GNU General Public License (GPL). Anyone who is interested is
 * allowed to view, modify, and/or improve upon the code used to produce this
 * package, but any software generated using all or part of this code must be
 * released as free open-source software in turn. The GNU GPL can be viewed in
 * its entirety as in the following site: http://www.gnu.org/licenses/gpl.html
 */
package amesmarket;

/**
 * Configuration parameters.
 *
 * Parameters are public to make adapting the existing code quicker.
 *
 * Adapted from {@link AMESGUIFrame.AMESFrame}
 * @author Sean L. Mooney
 */
public class CaseFileData {

    public Object[][] nodeData;
    public Object[][] branchData;
    public GenData[] genData;
    public Object[][] lse3SecData;  // 3-sectional 8-hour LSE data
    public Object[][] lseData;      // Combine 3-sectional to 24-hour LSE data
    public Object[][] lseSec1Data;  // First 8-hour LSE data
    public Object[][] lseSec2Data;  // Second 8-hour LSE data
    public Object[][] lseSec3Data;  // Third 8-hour LSE data
    public Object[][][] lsePriceSensitiveDemand;
    public Object[][] lseHybridDemand;

    public int iNodeData;
    public int iBranchData;
    public int iGenData;
    public int iLSEData;
    public boolean bPriceSensitiveDemand;
    public boolean bHybridDemand;
    public boolean isPU;
    public double baseS;
    public double baseV;

    // Learning and action domain parameters
    public double Default_Cooling;
    public double Default_Experimentation;
    public double Default_InitPropensity;
    public int Default_M1;
    public int Default_M2;
    public int Default_M3;
    public double Default_RI_MAX_Lower;
    public double Default_RI_MAX_Upper;
    public double Default_RI_MIN_C;
    public double Default_Recency;
    public double Default_SlopeStart;
    public int Default_iRewardSelection;
    public double Cooling;
    public double Experimentation;
    public double InitPropensity;
    public int M1;
    public int M2;
    public int M3;
    public double RI_MAX_Lower;
    public double RI_MAX_Upper;
    public double RI_MIN_C;
    public double Recency;
    public double SlopeStart;
    public int iRewardSelection;
    public double[][] genLearningData;
    public boolean bGenLearningDataSet = false;

    public long RandomSeed;
    public int iMaxDay;
    public double dThresholdProbability;
    public double dDailyNetEarningThreshold;
    public double dGenPriceCap;
    public double dLSEPriceCap;
    public int iStartDay;
    public int iCheckDayLength;
    public double dActionProbability;
    public int iLearningCheckStartDay;
    public int iLearningCheckDayLength;
    public double dLearningCheckDifference;
    public int iDailyNetEarningStartDay;
    public int iDailyNetEarningDayLength;
    public boolean bMaximumDay = true;
    public boolean bThreshold = true;
    public boolean bDailyNetEarningThreshold = false;
    public boolean bActionProbabilityCheck = false;
    public boolean bLearningCheck = false;

    public CaseFileData() {
        RandomSeed = 695672061;
        iMaxDay = 50;
        dThresholdProbability = 0.999;
        dDailyNetEarningThreshold = 10.0;
        dGenPriceCap = 1000.0;
        dLSEPriceCap = 0.0;
        iStartDay = 1;
        iCheckDayLength = 5;
        dActionProbability = 0.001;
        iLearningCheckStartDay = 1;
        iLearningCheckDayLength = 5;
        dLearningCheckDifference = 0.001;
        iDailyNetEarningStartDay = 1;
        iDailyNetEarningDayLength = 5;
    }

    /**
     * Model the GenData section of the input file.
     * @author Sean L. Mooney
     *
     */
    public static class GenData{

      /**
       * @param name
       * @param id
       * @param atBus
       * @param sCost
       * @param a
       * @param b
       * @param capL
       * @param capU
       * @param initMoney
       */
      public GenData(String name, int id, int atBus, double sCost, double a,
          double b, double capL, double capU, double initMoney) {
        this.name = name;
        this.id = id;
        this.atBus = atBus;
        this.sCost = sCost;
        this.a = a;
        this.b = b;
        this.capL = capL;
        this.capU = capU;
        this.initMoney = initMoney;
      }

      /**
       * Creates a copy of the parameters to a double array.
       *
       * Utility method to make loading data into the AMESMarket easy.
       * @return
       */
      public Object[] asArray(){
        return new Object[] {
            name, id, atBus, sCost, a, b, capL, capU, initMoney
        };
      }

      public final String name;
      public final int id;
      public final int atBus;
      public final double sCost;
      public final double a;
      public final double b;
      public final double capL;
      public final double capU;
      public final double initMoney;
    }
}
