package edu.iastate.jrelm.similarity;

import edu.iastate.jrelm.core.Action;
import edu.iastate.jrelm.core.SimpleAction;


/**
 * This is a dissimilarity measure based soley on the position of actions in a
 * one-dimensional collection, such as an IntegerIndexedDomain. Two actions 
 * are said to be similar if their indexes are close to each other. How close 
 * they need to be is determined by the given threshold value.
 * <P>
 * For example, if threshold of 2 is given, then the index of an action must 
 * be within +2 or -2 of the index of another action for them to be 
 * considered similar.
 * <P>
 * This similarity measure is useful for collections where the actions have 
 * already been organized or pre-sorted according to their similarity. 
 * <P>
 * Note: This SimilarityMeasure does not check to see if both actions are from 
 * the same domain. Comparing two SimpleActions from different domains may result 
 * in undesirable results.
 * 
 * @author Charles Gieseler
 *
 * @param <A> - type of Action this measure will compare. Must be a derivitive 
 * of SimpleAction
 */
public class SimpleDissimilarity<A extends Action> 
		implements DissimilarityMeasure<A, Double>{

	//
	private int threshold;
	
	/**
	 * Construct a SimpleSimilarity measure with the given index distance threshold.
	 * See above for explaination of threshold.
	 * 
	 * @param indexThreshold - the max range that two indexes can be separated by 
	 *						   and still be considered similar
	 */
	public SimpleDissimilarity(int indexThreshold){
		threshold = indexThreshold;
	}
	
	/**
	 * Constructs a SimpleSimilarity measure with a threshold of 3. See above for explaination
	 * of threshold.
	 *
	 */
	public SimpleDissimilarity(){
		this(3);
	}
	
	/**
	 * Accepts two Action and checks to see if their indexes are close
	 * to each other. If the indexes are within the set threshold value, then this returns 
	 * a Boolean with a value of true. Otherwise, the returned Boolean has a value of false.
	 * <P>
	 * Note: This SimilarityMeasure does not check to see if both actions are from the same
	 * domain. Comparing two SimpleActions from different domains may result 
	 * in inaccurate results. 
	 * <P>
	 * Also, this method does not operate over circular indexing. If the threshold 
	 * extends beyond the bounds of the first action index, this will not 
	 * "cirle around" to include the last action index within the treshold.
	 * 
	 */
	public Double dissimilarity(A action1, A action2) {
		//TODO: not yet implemented
		return null;
//		SimpleAction act1, act2;
//		
//		try{
//			act1 = (SimpleAction) action1;
//			act2 = (SimpleAction) action2;
//		}catch(ClassCastException cce){
//			throw new IllegalArgumentException("SimpleSimilarity.similarity() may only be used " +
//					"to compare Actions of type SimpleAction.");
//		}
//		
//		boolean areSimilar = false;
//		int index1 = ((Integer)act1.getID()).intValue();
//		int index2 = ((Integer)act2.getID()).intValue();
//		
//		//Check to see if these action indexes are within range of each other
//		if((index2 >= (index1-threshold)) && (index2 <= (index1+threshold))){
//			areSimilar = true;
//		}else{
//			areSimilar = false;
//		}
//		
//		return new Boolean(areSimilar);
	}

}
